package es.uca.xmleye.gui;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;

import static es.uca.xmleye.SearchScope.*;
import es.uca.xmleye.SearchRequest;
import es.uca.xmleye.SearchScope;
import es.uca.xmleye.gui.events.SearchListener;
import es.uca.xmleye.gui.util.PressButtonAction;

/**
 * Diálogo Buscar.
 */
public class WndFind extends JDialog {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1340475896257659887L;

	private static ResourceBundle res;

	private JLabel etiqBuscar;
	private JTextField txtBuscar;
	private JCheckBox chkPalabraCompleta;
	private JCheckBox chkCoincideMayusMinus;
	private JRadioButton radDesdeCursor;
	private JRadioButton radDesdeRaiz;
	private JRadioButton radHastaCursor;
	private JButton btnBuscar;
	private JButton btnCerrar;
	private AbstractAction accBuscar;
	private AbstractAction accCerrar;
	private ButtonGroup grpAlcanceBusqueda;

	private ArrayList<SearchListener> obsBusqueda = new ArrayList<SearchListener>();

	static {
		res = ResourceBundle.getBundle(
				"es.uca.xmleye.gui.l10n.ResWndFind", Locale.getDefault());
	}

	/**
	 * Crea un nuevo diálogo Buscar y lo muestra en pantalla.
	 * 
	 * @param padre
	 *            JFrame padre de este diálogo.
	 */
	public WndFind(JFrame padre) {
		super(padre, res.getString("wnd_find"));

		inicPropiedades();
		inicAcciones();
		inicComponentes();
		inicObservadores();

		pack();
		setLocationRelativeTo(padre);
		setVisible(true);
	}

	/**
	 * Punto de lanzamiento independiente para probar la interfaz gráfica.
	 */
	public static void main(String[] args) {
		final WndFind ventana = new WndFind(null);
		ventana.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		ventana.btnCerrar.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ventana.dispose();
			}
		});
	}

	/**
	 * Registra un observador de peticiones de búsqueda.
	 * 
	 * @param observador
	 *            Nuevo observador de búsquedas.
	 */
	public void agregarObservadorBusqueda(SearchListener observador) {
		obsBusqueda.add(observador);
	}

	/**
	 * Retira un observador de peticiones de búsqueda.
	 * 
	 * @param observador
	 *            Observador de búsquedas a retirar.
	 */
	public void retirarObservadorBusqueda(SearchListener observador) {
		obsBusqueda.remove(observador);
	}

	/** * METODOS PROTEGIDOS ************************** */

	/**
	 * Inicializa las propiedades del diálogo.
	 */
	protected void inicPropiedades() {
		setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
		setFocusTraversalPolicy(new VentanaBuscarFocusTraversalPolicy());
		setResizable(false);
		getContentPane().setLayout(new GridBagLayout());
	}

	/**
	 * Inicializa las acciones del diálogo.
	 */
	protected void inicAcciones() {
		accBuscar = new AccionBuscar();
		accCerrar = new AccionCerrar();
	}

	/**
	 * Crea los componentes, establece sus propiedades y los sitúa en el
	 * diálogo.
	 */
	protected void inicComponentes() {
		GridBagConstraints c = new GridBagConstraints();

		etiqBuscar = new JLabel(res.getString("lbl_find"));
		txtBuscar = new JTextField();
		chkPalabraCompleta = new JCheckBox(res.getString("chk_wholestring"));
		chkCoincideMayusMinus = new JCheckBox(res.getString("chk_matchcase"));
		radDesdeCursor = new JRadioButton(res.getString("rad_fromcursor"));
		radDesdeRaiz = new JRadioButton(res.getString("rad_fromroot"));
		radHastaCursor = new JRadioButton(res.getString("rad_uptocursor"));
		btnBuscar = new JButton(accBuscar);
		btnCerrar = new JButton(accCerrar);

		etiqBuscar.setLabelFor(txtBuscar);
		etiqBuscar.setDisplayedMnemonic(((Character) res
				.getObject("lbl_find_m")).charValue());
		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(12, 12, 11, 12);
		c.weightx = 0.0;
		c.weighty = 0.0;
		c.anchor = GridBagConstraints.LINE_START;
		getContentPane().add(etiqBuscar, c);

		txtBuscar.requestFocusInWindow();
		c.gridx = 1;
		c.gridwidth = 3;
		c.insets = new Insets(12, 0, 11, 11);
		c.weightx = 1.0;
		c.fill = GridBagConstraints.HORIZONTAL;
		getContentPane().add(txtBuscar, c);

		chkPalabraCompleta.setMnemonic(((Integer) res
				.getObject("chk_wholestring_m")).intValue());
		c.gridy = 1;
		c.gridwidth = 1;
		c.insets = new Insets(0, 0, 5, 11);
		c.weightx = 0.0;
		getContentPane().add(chkPalabraCompleta, c);

		chkCoincideMayusMinus.setMnemonic(((Integer) res
				.getObject("chk_matchcase_m")).intValue());
		c.gridy = 2;
		getContentPane().add(chkCoincideMayusMinus, c);

		radDesdeRaiz.setMnemonic(((Integer) res.getObject("rad_fromroot_m"))
				.intValue());
		c.gridy = 1;
		c.gridx = 2;
		c.gridwidth = 2;
		c.insets = new Insets(0, 0, 5, 11);
		getContentPane().add(radDesdeRaiz, c);

		radDesdeCursor
				.setMnemonic(((Integer) res.getObject("rad_fromcursor_m"))
						.intValue());
		c.gridy = 2;
		getContentPane().add(radDesdeCursor, c);

		radHastaCursor
				.setMnemonic(((Integer) res.getObject("rad_uptocursor_m"))
						.intValue());
		c.gridy = 3;
		getContentPane().add(radHastaCursor, c);

		btnBuscar.setDefaultCapable(true);
		getRootPane().setDefaultButton(btnBuscar);
		c.gridwidth = 1;
		c.gridy = 4;
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(17, 11, 11, 5);
		c.anchor = GridBagConstraints.LINE_END;
		c.weightx = 0.5;
		c.weighty = 1.0;
		getContentPane().add(btnBuscar, c);

		btnCerrar.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				KeyStroke.getKeyStroke("ESCAPE"), "click");
		btnCerrar.getActionMap().put("click", new PressButtonAction(btnCerrar));
		c.gridx = 3;
		c.insets = new Insets(17, 0, 11, 11);
		c.anchor = GridBagConstraints.LINE_START;
		getContentPane().add(btnCerrar, c);

		grpAlcanceBusqueda = new ButtonGroup();
		grpAlcanceBusqueda.add(radDesdeCursor);
		grpAlcanceBusqueda.add(radDesdeRaiz);
		grpAlcanceBusqueda.add(radHastaCursor);
		grpAlcanceBusqueda.setSelected(radDesdeRaiz.getModel(), true);
	}

	/**
	 * Inicializa los observadores de eventos a nivel de ventana.
	 */
	protected void inicObservadores() {
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowActivated(WindowEvent e) {
				txtBuscar.requestFocusInWindow();
			}
		});
	}

	/** CLASES ANIDADAS ******************************* */

	/**
	 * Acción de búsqueda.
	 */
	private class AccionBuscar extends AbstractAction {

		/**
		 * 
		 */
		private static final long serialVersionUID = 2372374087106369165L;

		/**
		 * Crea una nueva acción de búsqueda. El texto de la acción está
		 * internacionalizado mediante el <code>ResourceBundle</code> de
		 * <code>WndFind</code>.
		 */
		public AccionBuscar() {
			super(res.getString("act_find"));
			putValue(SHORT_DESCRIPTION, res.getString("act_find_d"));
		}

		/**
		 * Envía a los observadores un evento de petición de búsqueda. Se
		 * construye a partir de la información de los componentes del diálogo.
		 * 
		 * @param e
		 *            Evento a tratar.
		 */
		public void actionPerformed(ActionEvent e) {
			SearchScope alcance;

			// Validamos la entrada: debe de haber una clave de búsqueda
			if (txtBuscar.getText().length() == 0) {
				MessageManagerSingleton.getInstancia().mostrarInfo(
						res.getString("err_searchkey_notfound"));
				return;
			}

			{
				final ButtonModel bMod = grpAlcanceBusqueda.getSelection();

				if (radDesdeCursor.getModel() == bMod) {
					alcance = ALCANCE_SIGUIENTES_SUBARBOL;
				} else if (radDesdeRaiz.getModel() == bMod) {
					alcance = ALCANCE_DOCUMENTO;
					grpAlcanceBusqueda.setSelected(bMod, false);
					grpAlcanceBusqueda.setSelected(radDesdeCursor.getModel(),
							true);
				} else if (radHastaCursor.getModel() == bMod) {
					alcance = ALCANCE_ANTERIORES_ANCESTROS;
				} else {
					MessageManagerSingleton.getInstancia().mostrarError(
							res.getString("err_scope_not_exist"));
					return;
				}
			}

			// Este diálogo no sabe nada acerca de sobre qué árbol
			// exactamente se va a lanzar la búsqueda
			final SearchRequest ev = new SearchRequest();
			ev.setClaveBusqueda(txtBuscar.getText());
			ev.setPalabraCompleta(chkPalabraCompleta.isSelected());
			ev.setCoincidenMayusMinus(chkCoincideMayusMinus.isSelected());
			ev.setAlcance(alcance);

			// Copiamos la lista de observadores para evitar que nos
			// invaliden la lista durante el lanzamiento de los eventos (ver
			// http://www.swingwiki.org/best:thread-safe_listener_processing)
			ArrayList<SearchListener> listeners = new ArrayList<SearchListener>(
					obsBusqueda);
			for (ListIterator<SearchListener> iterFL = listeners
					.listIterator(); iterFL.hasNext();) {
				iterFL.next().busquedaLanzada(ev);
			}
		}
	}

	/**
	 * Acción de cierre del diálogo.
	 */
	private class AccionCerrar extends AbstractAction {
		/**
		 * 
		 */
		private static final long serialVersionUID = -1168858602176068837L;

		public AccionCerrar() {
			super(res.getString("act_close"));
			putValue(SHORT_DESCRIPTION, res.getString("act_close_d"));
		}

		/**
		 * Oculta el diálogo.
		 */
		public void actionPerformed(ActionEvent e) {
			setVisible(false);
		}
	}

	/**
	 * Política de cambio de foco personalizada para este diálogo.
	 */
	private class VentanaBuscarFocusTraversalPolicy extends
			FocusTraversalPolicy {
		@Override
		public Component getComponentAfter(Container raiz, Component c) {
			if (c.equals(txtBuscar)) {
				return chkPalabraCompleta;
			} else if (c.equals(chkPalabraCompleta)) {
				return chkCoincideMayusMinus;
			} else if (c.equals(chkCoincideMayusMinus)) {
				return radDesdeCursor;
			} else if (c.equals(radDesdeCursor)) {
				return radDesdeRaiz;
			} else if (c.equals(radDesdeRaiz)) {
				return radHastaCursor;
			} else if (c.equals(radHastaCursor)) {
				return btnBuscar;
			} else if (c.equals(btnBuscar)) {
				return btnCerrar;
			} else if (c.equals(btnCerrar)) {
				return txtBuscar;
			} else {
				return txtBuscar;
			}
		}

		@Override
		public Component getComponentBefore(Container raiz, Component c) {
			if (c.equals(txtBuscar)) {
				return btnCerrar;
			} else if (c.equals(chkPalabraCompleta)) {
				return txtBuscar;
			} else if (c.equals(chkCoincideMayusMinus)) {
				return chkPalabraCompleta;
			} else if (c.equals(radDesdeCursor)) {
				return chkCoincideMayusMinus;
			} else if (c.equals(radDesdeRaiz)) {
				return radDesdeCursor;
			} else if (c.equals(radHastaCursor)) {
				return radDesdeRaiz;
			} else if (c.equals(btnBuscar)) {
				return radHastaCursor;
			} else if (c.equals(btnCerrar)) {
				return btnBuscar;
			} else {
				return btnBuscar;
			}
		}

		@Override
		public Component getDefaultComponent(Container raiz) {
			return txtBuscar;
		}

		@Override
		public Component getFirstComponent(Container raiz) {
			return txtBuscar;
		}

		@Override
		public Component getLastComponent(Container raiz) {
			return btnCerrar;
		}

	}
}
