package es.uca.xmleye.presentation;

/*
 Copyright (C) 2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import javax.swing.ListModel;

import es.uca.xmleye.formats.FormatDescriptor;
import es.uca.xmleye.formats.FormatDescriptorException;
import es.uca.xmleye.formats.FormatDescriptorRepository;
import es.uca.xmleye.gui.Application;


/**
 * Modelo de presentación para la ventana de formatos de documentos aceptados.
 * 
 * @author Antonio García Domínguez
 * @version 1.0
 */
public class WndFormatsPresentationModel {
	private FormatDescriptorRepository repositorio;

	// Identificador secuencial del elemento seleccionado
	private int seleccion;

	/**
	 * Recommended constructor, through Dependency Injection.
	 */
	public WndFormatsPresentationModel(FormatDescriptorRepository repositorio)
			throws FormatDescriptorException {
		inicializar(repositorio);
	}

	/**
	 * For when we just want to test out the UI, and can't count on the
	 * assembler.
	 */
	public WndFormatsPresentationModel() throws FormatDescriptorException {
		inicializar(new FormatDescriptorRepository(
				new String[] { Application.REPO_GLOBAL }));
	}

	private void inicializar(FormatDescriptorRepository repositorio) {
		this.repositorio = repositorio;
		seleccion = repositorio.getSize() > 0 ? 0 : -1;
	}

	/** Devuelve el modelo a utilizar para construir la lista de formatos. */
	public ListModel getModeloLista() {
		return repositorio;
	}

	/** Devuelve el texto a mostrar en el campo Orden de Edición. */
	public String getTextoOrdenEdicion() {
		return seleccion != -1 ? ((FormatDescriptor) repositorio
				.getElementAt(seleccion)).getPatronOrdenEdicion() : "";
	}

	/** Devuelve el texto a mostrar en el campo Orden de Importación. */
	public String getTextoOrdenImportacion() {
		return seleccion != -1 ? ((FormatDescriptor) repositorio
				.getElementAt(seleccion)).getPatronOrdenImportacion() : "";
	}

	/** Devuelve el texto a mostrar en el campo Extensiones. */
	public String getTextoExtensiones() {
		final String[] exts = seleccion != -1 ? ((FormatDescriptor) repositorio
				.getElementAt(seleccion)).getExtensiones() : null;

		if (exts == null)
			return "";

		StringBuffer sBuf = new StringBuffer();
		if (exts.length > 0) {
			sBuf.append(exts[0]);
			for (int i = 1; i < exts.length; ++i) {
				sBuf.append(", ");
				sBuf.append(exts[i]);
			}
		}

		return sBuf.toString();
	}

	/** Devuelve el texto a mostrar en el campo Nombre. */
	public String getTextoNombre() {
		return seleccion != -1 ? ((FormatDescriptor) repositorio
				.getElementAt(seleccion)).getNombre() : "";
	}

	/**
	 * Devuelve la posición del elemento a seleccionar (-1 si no debe haber
	 * selección).
	 */
	public int getSeleccion() {
		return seleccion;
	}

	/** El texto del campo Orden de Edición ha cambiado. */
	public void textoOrdenEdicionCambiado(String nuevoTexto)
			throws FormatDescriptorException {
		if (seleccion == -1 || nuevoTexto.equals(getTextoOrdenEdicion()))
			return;

		((FormatDescriptor) repositorio.getElementAt(seleccion))
				.setPatronOrdenEdicion(nuevoTexto);
	}

	/** El texto del campo Orden de Importación ha cambiado. */
	public void textoOrdenImportacionCambiado(String nuevoTexto)
			throws FormatDescriptorException {
		if (seleccion == -1 || nuevoTexto.equals(getTextoOrdenImportacion()))
			return;
		((FormatDescriptor) repositorio.getElementAt(seleccion))
				.setPatronOrdenImportacion(nuevoTexto);
	}

	/** El texto del campo Extensiones ha cambiado. */
	public void textoExtensionesCambiado(String nuevoTexto)
			throws FormatDescriptorException {
		if (seleccion == -1 || nuevoTexto.equals(getTextoExtensiones()))
			return;
		final String[] exts = nuevoTexto.split("\\s*,\\s*");
		((FormatDescriptor) repositorio.getElementAt(seleccion))
				.setExtensiones(exts);
	}

	/** El texto del campo Nombre ha cambiado. */
	public void textoNombreCambiado(String nuevoTexto)
			throws FormatDescriptorException {
		if (seleccion == -1 || nuevoTexto.equals(getTextoNombre()))
			return;
		((FormatDescriptor) repositorio.getElementAt(seleccion))
				.setNombre(nuevoTexto);
	}

	/** La selección ha cambiado. */
	public void seleccionCambiada(int nuevaPos) {
		seleccion = nuevaPos;
	}

	/** Se ha pulsado Aceptar. */
	public void cambiosConfirmados() throws FormatDescriptorException {
		repositorio.guardar();
	}

	/** Se ha pulsado Cancelar. */
	public void cambiosCancelados() throws FormatDescriptorException {
		repositorio.recargar();
	}

	/** Se ha pulsado Restaurar. */
	public void restauracionSolicitada() throws FormatDescriptorException {
		if (seleccion == -1)
			return;
		((FormatDescriptor) repositorio.getElementAt(seleccion)).restaurar();
	}

}