package es.uca.xmleye.util;

/*
  Copyright (C) 2005-2008 Antonio García Domínguez

  This program is free software; you can redistribute it
  and/or modify it under the terms of the GNU General Public
  License as published by the Free Software Foundation;
  either version 2 of the License, or (at your option) any
  later version.

  This program is distributed in the hope that it will be
  useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
  PURPOSE. See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public
  License along with this program; if not, write to the Free
  Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA 02111-1307 USA
*/

/**
 * Filter which appends all lines to an in-memory buffer. Useful for
 * saving to memory the contents of the stream being processed as they
 * are produced.
 *
 * @author Antonio García Domínguez
 */
public class SaveToMemoryLineFilter implements LineFilter {
  StringBuffer sBuffer = new StringBuffer();
  public static final String LINE_SEPARATOR = "\n";

  public void processLine(String line) throws Exception {
    sBuffer.append(line);
    sBuffer.append(LINE_SEPARATOR);
  }

  /**
   * Returns all the lines processed as a single long string, joined by
   * {@link LINE_SEPARATOR}.
   *
   * @return String with all the lines read concatenated.
   */
  public String getSavedContents() {
    return sBuffer.toString();
  }

  public void eofReached() throws Exception {
    // empty
  }
}
