package es.uca.xmleye.xslt;

/*
 Copyright (C) 2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import es.uca.xmleye.xslt.PreprocessingStylesheet;
import es.uca.xmleye.xslt.ViewStylesheet;
import es.uca.xmleye.xslt.StylesheetRepository;
import junit.framework.*;
import junit.textui.*;

/**
 * Tests de unidad para Transformaciones.
 */
public class StylesheetRepositoryTest extends TestCase {
	private StylesheetRepository repositorio;

	/** Crea un nuevo caso de prueba. */
	public StylesheetRepositoryTest(String nombre) {
		super(nombre);
	}

	/** Crea el conjunto de pruebas para JUnit. */
	public static Test suite() {
		return new TestSuite(StylesheetRepositoryTest.class);
	}

	/** Lanza el caso de prueba mediante JUnit. */
	public static void main(String[] args) {
		TestRunner.run(suite());
	}

	/** *** PRUEBAS *********************************** 
	 * @throws TransformationException */

	public void testHojasVistaHalladas() throws TransformationException {
		final String[] nombreHojas = new String[] { "xml", "xmlSource" };

		// Comprobamos que han sido halladas con éxito
		// y que tienen los datos correctos
		for (String nombreHoja : nombreHojas) {
			ViewStylesheet hoja = repositorio
					.getHojaVisualizacion(nombreHoja);

			// Comprobamos que existe en el repositorio y que tiene los datos
			// correctos
			assertNotNull("No se ha encontrado la hoja " + nombreHoja, hoja);
			assertEquals(hoja.getNombre(), nombreHoja);
			assertEquals(hoja.getRutaXSLT(), repositorio
					.getHojaPrincipalVista());

			// Comprobamos que se compila sin problemas
			hoja.getTranslet();
		}
	}

	public void testHojasPreprocesadoHalladas() throws TransformationException {
		final String[] nombreHojas = new String[] { "xml" };

		// Comprobamos que han sido halladas con éxito
		// y que tienen los datos correctos
		for (String nombreHoja : nombreHojas) {
			PreprocessingStylesheet hoja = repositorio.getHojaPreprocesado(nombreHoja);

			// Comprobamos que existe en el repositorio y que tiene los datos
			// correctos
			assertNotNull("No se ha encontrado la hoja " + nombreHoja, hoja);
			assertEquals(hoja.getNombre(), nombreHoja);
			assertEquals(hoja.getRutaXSLT(), repositorio
					.getHojaPrincipalPreprocesado());

			// Comprobamos que se compila sin problemas
			hoja.getTranslet();
		}
	}

	@Override
	public void setUp() throws StylesheetRepositoryException {
		repositorio = new StylesheetRepository("build/xslt");
	}

	@Override
	public void tearDown() {
	}

}
